package cd.casic.ci.process.process.service.target.impl;


import cd.casic.ci.process.dal.req.target.*;
import cd.casic.ci.process.dal.resp.target.TargetManagerResp;
import cd.casic.ci.process.process.converter.TargetConverter;
import cd.casic.ci.process.process.dal.pipeline.TargetManagerDao;
import cd.casic.ci.process.process.dal.pipeline.TargetVersionDao;
import cd.casic.ci.process.process.dataObject.base.BaseIdReq;
import cd.casic.ci.process.process.dataObject.target.TargetManager;
import cd.casic.ci.process.process.dataObject.target.TargetVersion;
import cd.casic.ci.process.process.service.target.TargetManagerService;
import cd.casic.ci.process.util.SftpUploadUtil;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.framework.commons.util.object.BeanUtils;
import cd.casic.framework.security.dal.user.AdminUserDO;
import cd.casic.framework.tenant.core.service.AdminUserServiceImpl;
import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName TargetManagerServiceImpl
 * @Date: 2025/5/17 15:19
 * @Description:
 */
@Service
@Slf4j
public class TargetManagerServiceImpl extends ServiceImpl<TargetManagerDao, TargetManager> implements TargetManagerService {

    @Resource
    private TargetVersionServiceImpl targetVersionService;

    @Resource
    private TargetManagerDao targetManagerDao;

    @Resource
    private TargetVersionDao targetVersionDao;

    @Resource
    private AdminUserServiceImpl adminUserService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createTarget(TargetManagerCreateReq req) {
        TargetManager targetManager = new TargetManager();
        BeanUtils.copyProperties(req, targetManager);
        this.save(targetManager);

        //保存第一个版本
        TargetVersion targetVersion = new TargetVersion();
        BeanUtil.copyProperties(req,targetVersion);
        targetVersion.setTargetId(targetManager.getId());
        targetVersionService.save(targetVersion);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateTarget(TargetManagerUpdateReq req) {
        if (StringUtils.isEmpty(req.getId())){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"id不能为空");
        }

        TargetManager targetManager = targetManagerDao.selectById(req.getId());
        if (ObjectUtils.isEmpty(targetManager)){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"数据错误，请联系管理员");
        }

        BeanUtils.copyProperties(req,targetManager);
        targetManagerDao.updateById(targetManager);

        //根据版本号和targetId查询是修改当前版本的信息还是新增新版本
        QueryWrapper<TargetVersion> wrapper = new QueryWrapper<>();
        wrapper.eq("target_id",targetManager.getId());
        wrapper.eq("version",req.getVersion());
        List<TargetVersion> versionList = targetVersionDao.selectList(wrapper);

        if (CollectionUtils.isEmpty(versionList)){
            //新增一条版本信息
            TargetVersion targetVersion = new TargetVersion();
            BeanUtil.copyProperties(req,targetVersion);
            targetVersion.setId(null);
            targetVersion.setTargetId(targetManager.getId());
            targetVersionService.save(targetVersion);
        }else {
            TargetVersion targetVersion = versionList.get(0);
            String versionId = targetVersion.getId();
            BeanUtils.copyProperties(req,targetVersion);
            targetVersion.setId(versionId);
            targetVersionService.updateById(targetVersion);
        }


    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteTarget(BaseIdReq req) {
        if (StringUtils.isEmpty(req.getId())){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"id不能为空");
        }

        TargetManager targetManager = targetManagerDao.selectById(req.getId());
        if (ObjectUtils.isEmpty(targetManager)){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"数据错误，请联系管理员");
        }
        targetManagerDao.deleteById(targetManager.getId());

        //删除对应的version版本
        List<TargetVersion> versionList = targetVersionService.selectListByManagerId(targetManager.getId());
        List<String> versionIdList = versionList.stream().map(TargetVersion::getId).toList();
        targetVersionDao.deleteByIds(versionIdList);
    }

    @Override
    public PageResult<TargetManagerResp> findTargetPage(TargetQueryReq query) {
        Page<TargetManagerResp> respPage = new Page<>();

        QueryWrapper<TargetManager> wrapper = new QueryWrapper<>();
        if (!ObjectUtils.isEmpty(query.getId())){
            wrapper.eq("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getIdList())){
            wrapper.in("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getTargetName())){
            wrapper.like("target_name",query.getTargetName());
        }
        if (!ObjectUtils.isEmpty(query.getCreator())){
            wrapper.eq("creator",query.getCreator());
        }

        Page<TargetManager> targetManagerPage = targetManagerDao.selectPage(new Page<>(query.getPageNo(), query.getPageSize()), wrapper);

        if (ObjectUtils.isEmpty(targetManagerPage)){
            return new PageResult<>();
        }

        List<TargetManagerResp> respList = TargetConverter.INSTANCE.toRespList(targetManagerPage.getRecords());
        respList.forEach(this::setUserName);

        respPage.setRecords(respList);
        respPage.setTotal(targetManagerPage.getTotal());
        respPage.setCurrent(targetManagerPage.getCurrent());
        respPage.setSize(targetManagerPage.getSize());
        PageResult<TargetManagerResp> pageResult = new PageResult<>(respList,targetManagerPage.getTotal());
        return pageResult;
    }

    private void setUserName(TargetManagerResp targetManagerResp) {
        if (!StringUtils.isEmpty(targetManagerResp.getCreator())){
            AdminUserDO user = adminUserService.getUser(Long.valueOf(targetManagerResp.getCreator()));
            if (!ObjectUtils.isEmpty(user)){
                targetManagerResp.setCreatorName(user.getUsername());
            }
        }

        if (!StringUtils.isEmpty(targetManagerResp.getUpdater())){
            AdminUserDO user = adminUserService.getUser(Long.valueOf(targetManagerResp.getUpdater()));
            if (!ObjectUtils.isEmpty(user)){
                targetManagerResp.setUpdaterName(user.getUsername());
            }
        }
    }

    @Override
    public List<TargetManagerResp> findTargetList(TargetQueryReq query) {
        QueryWrapper<TargetManager> wrapper = new QueryWrapper<>();
        if (!ObjectUtils.isEmpty(query.getId())){
            wrapper.eq("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getIdList())){
            wrapper.in("id",query.getId());
        }
        if (!ObjectUtils.isEmpty(query.getTargetName())){
            wrapper.like("target_name",query.getTargetName());
        }
        if (!ObjectUtils.isEmpty(query.getCreator())){
            wrapper.eq("creator",query.getCreator());
        }
        List<TargetManager> targetManagerList = targetManagerDao.selectList(wrapper);
        if (ObjectUtils.isEmpty(targetManagerList)){
            return new ArrayList<>(0);
        }

        List<TargetManagerResp> respList = TargetConverter.INSTANCE.toRespList(targetManagerList);
        respList.forEach(this::setUserName);

        return respList;
    }

    @Override
    public String fileUpload(TargetUploadReq req) throws SftpUploadUtil.SftpUploadException {
        String filePath = req.getRemoteDir() + UUID.randomUUID();

        SftpUploadUtil.uploadFileSftp(
                req.getRemoteHost(),
                req.getRemotePort(),
                req.getUsername(),
                req.getPassword(),
                req.getSshKeyPath(),
                req.getFile(),
                filePath,
                req.getRemoteFileName()
        );
        return filePath;
    }

    @Override
    public void createVersion(TargetVersionCreateReq req) {
        TargetVersion targetVersion = new TargetVersion();
        BeanUtil.copyProperties(req,targetVersion);
        targetVersionService.save(targetVersion);
    }

    @Override
    public void deleteVersion(BaseIdReq req) {
        targetManagerDao.deleteById(req.getId());
    }
}
